--[[ ---------------------------------------------------------------------------

Acheron: death reports

Acheron is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Acheron is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with Acheron.	If not, see <http://www.gnu.org/licenses/>.

----------------------------------------------------------------------------- ]]

--[[ ---------------------------------------------------------------------------
	 Addon and libraries
----------------------------------------------------------------------------- ]]

local L = LibStub("AceLocale-3.0"):GetLocale("Acheron")
local C = LibStub("AceConfigDialog-3.0")


--[[ ---------------------------------------------------------------------------
	 Local function copies
----------------------------------------------------------------------------- ]]

local type               = _G.type
local tostring           = _G.tostring


--[[ ---------------------------------------------------------------------------
	 Create dropdowns
----------------------------------------------------------------------------- ]]

local logLevelsDropdown = {}

if Acheron.logLevels then
	for logname,id in pairs(Acheron.logLevels) do
		logLevelsDropdown[id] = L[logname]
	end
end

--[[ ---------------------------------------------------------------------------
	 Config options
----------------------------------------------------------------------------- ]]
Acheron.options = {
	name = "Acheron",
	handler = Acheron,
	type = "group",
	args = {
		general = {
			type = "group",
			name = L["General"],
			order = 0,
			set = "SetProfileParam",
			get = "GetProfileParam",
			cmdHidden = true,
			order = 0,
			args = {
				history = {
					type = "range",
					name = L["History"],
					desc = L["The amount of history, in seconds, of combat log to keep per player"],
					min = 10,
					max = 60,
					step = 1,
					bigStep = 1,
					order = 1,
				},
				numreports = {
					type = "range",
					name = L["Number of Reports"],
					desc = L["The numer of death reports to keep per player, 0 for no limit"],
					min = 0,
					max = 10,
					step = 1,
					bigStep = 1,
					order = 2,
				},
				logLevel = {
					type = "select",
					name = L["Log Level"],
					desc = L["Determines the amount of output from the addon"],
					order = 99,
					values = logLevelsDropdown,
					disabled = function() return not LibStub("LibLogger-1.0", true) end,
				},
			},
		},
	},
}

Acheron.options.args[L["config"]] = {
	type = "execute",
	name = L["config"],
	desc = L["Toggle the configuration dialog"],
	func = "ToggleConfigDialog",
	guiHidden = true,
	order = 1,
}


Acheron.options.args[L["clear"]] = {
	type = "execute",
	name = L["clear"],
	desc = L["Clear all reports"],
	func = "ClearReports",
	guiHidden = true,
	order = 1,
}


--[[ ---------------------------------------------------------------------------
	 Config defaults
----------------------------------------------------------------------------- ]]
Acheron.defaults = {
	profile = {
		history = 30,
		numreports = 0,
		reporttime = 10,
		reportchannel = "raid",
		reportthreshold = 300,
		logLevel = 0,
	},
}


--[[ ---------------------------------------------------------------------------
	 Toggle the config window
----------------------------------------------------------------------------- ]]
function Acheron:ToggleConfigDialog()

   local frame = C.OpenFrames["Acheron"]

   if frame then
	  C:Close("Acheron")
   else
	  C:Open("Acheron")
	  self:SetStatusText(format(L["Profile: %s"], self.db:GetCurrentProfile()))
   end

end


--[[ ---------------------------------------------------------------------------
	 Set the text displayed at the bottom of the config window
----------------------------------------------------------------------------- ]]
function Acheron:SetStatusText(text)

   local frame = C.OpenFrames["Acheron"]

   if frame then
	  frame:SetStatusText(text)
   end

end


--[[ ---------------------------------------------------------------------------
	 Update config display on a profile change
----------------------------------------------------------------------------- ]]
function Acheron:OnProfileChanged(event, newdb)

   if event ~= "OnProfileDeleted" then
	  if LibStub("LibLogger-1.0", true) then self:SetLogLevel(self.db.profile.logLevel) end
	  self:SetStatusText(format(L["Profile: %s"], self.db:GetCurrentProfile()))
   end

end


--[[ ---------------------------------------------------------------------------
	 Set a saved variable config option
----------------------------------------------------------------------------- ]]
function Acheron:SetProfileParam(var, value)

   local varName = nil

   if type(var) == "string" then
	   varName = var
   else
	   varName = var[#var]
   end

   if self.trace then self:trace("Setting parameter %s to %s.", varName, tostring(value)) end

   self.db.profile[varName] = value

   if varName == "logLevel" and LibStub("LibLogger-1.0", true) then
	  self:SetLogLevel(value)
   end

end


--[[ ---------------------------------------------------------------------------
	 Get a saved variable config option
----------------------------------------------------------------------------- ]]
function Acheron:GetProfileParam(var) 

   local varName = nil

   if type(var) == "string" then
	   varName = var
   else
	   varName = var[#var]
   end

   if self.spam then self:spam("Getting parameter %s as %s", varName, tostring(self.db.profile[varName])) end

   return self.db.profile[varName]

end